import Flutter
import UIKit
import CoreLocation

public class SwiftBgLocationPlugin: NSObject, FlutterPlugin, CLLocationManagerDelegate {
    var channel: FlutterMethodChannel!
    var locationManager: CLLocationManager!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftBgLocationPlugin()
        instance.channel = FlutterMethodChannel(name: "bg_location", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: instance.channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        print("handle FlutterMethodCall " + call.method)
        switch(call.method) {
            case "getPlatformVersion":
                handleGetPlatformVersion(result: result)
                break
            case "startTracking":
                handleStartTracking(result: result)
                break
            case "stopTracking":
                handleStopTracking(result: result)
                break
            default:
                result("not implemented")
                break
        }
    }
    
    private func handleGetPlatformVersion(result: @escaping FlutterResult) {
        let platformInfo = "iOS " + UIDevice.current.systemVersion
        print("[BgLocation iOS] handleGetPlatformVersion success")
        result(platformInfo)
    }
    
    private func handleStartTracking(result: @escaping FlutterResult) {
        if !CLLocationManager.significantLocationChangeMonitoringAvailable() {
            result("not supported")
            return
        }
        
        locationManager = CLLocationManager()
        locationManager.delegate = self
        locationManager.showsBackgroundLocationIndicator = false
        locationManager.desiredAccuracy = kCLLocationAccuracyBest
        locationManager.allowsBackgroundLocationUpdates = true
        locationManager.pausesLocationUpdatesAutomatically = false
        locationManager.requestAlwaysAuthorization()
        locationManager.startMonitoringSignificantLocationChanges()
        //locationManager.startUpdatingLocation()
        print("[BgLocation iOS] handleStartTracking success")
        result("started")
    }
    
    private func handleStopTracking(result: @escaping FlutterResult) {
        if (locationManager == nil) {
            result("wasn't started")
            return
        }
        
        locationManager.stopMonitoringSignificantLocationChanges()
        locationManager = nil;
        print("[BgLocation iOS] handleStopTracking success")
        result("stopped")
    }
    
    // CLLocationManagerDelegate - handle new location
    public func locationManager(_ manager: CLLocationManager,  didUpdateLocations locations: [CLLocation]) {
       print("[BgLocation iOS] locationManager - handle new location: ...")
        
       let lastLocation = locations.last!
       let location = [
                    "speed": lastLocation.speed,
                    "altitude": lastLocation.altitude,
                    "latitude": lastLocation.coordinate.latitude,
                    "longitude": lastLocation.coordinate.longitude,
                    "accuracy": lastLocation.horizontalAccuracy,
                    "bearing": lastLocation.course,
                    "time": lastLocation.timestamp.timeIntervalSince1970 * 1000
                ] as [String : Any]

       channel.invokeMethod("onLocationUpdated", arguments: location)
    }
    
    // CLLocationManagerDelegate - handle error
    public func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
       print("[BgLocation iOS] locationManager - error");
       channel.invokeMethod("onLocationerror", arguments: "woah, error!")
       
        if let error = error as? CLError, error.code == .denied {
          manager.stopMonitoringSignificantLocationChanges()
          return
       }
    }
}
