import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class BgLocation {
  static const MethodChannel _channel = const MethodChannel('bg_location');

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<String?> startTracking() async {
    final String? result = await _channel.invokeMethod('startTracking');
    print('[BgLocation Plugin]: startTracking $result');
  }

  static Future<String?> stopTracking() async {
    final String? result = await _channel.invokeMethod('stopTracking');
    print('[BgLocation Plugin]: stopTracking $result');
  }

  static getLocationUpdates(Function(Location) onLocationUpdated) {
    _channel.setMethodCallHandler((MethodCall methodCall) async {
      print('[BgLocation Plugin]: ${methodCall.method} ${methodCall.arguments}');
      if (methodCall.method == 'onLocationUpdated') {
        var locationMap = Map.from(methodCall.arguments);
        var location = Location(
          latitude: locationMap['latitude'],
          longitude: locationMap['longitude'],
          altitude: locationMap['altitude'],
          accuracy: locationMap['accuracy'],
          bearing: locationMap['bearing'],
          speed: locationMap['speed'],
          time: locationMap['time'],
        );
        onLocationUpdated(location);
      }
    });
  }
}

class Location {
  double? latitude;
  double? longitude;
  double? altitude;
  double? bearing;
  double? accuracy;
  double? speed;
  double? time;

  Location(
      {@required this.longitude,
      @required this.latitude,
      @required this.altitude,
      @required this.accuracy,
      @required this.bearing,
      @required this.speed,
      @required this.time});

  toMap() {
    var result = {
      'latitude': latitude,
      'longitude': longitude,
      'altitude': altitude,
      'bearing': bearing,
      'accuracy': accuracy,
      'speed': speed,
      'time': time,
    };
    return result;
  }
}
